# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawFlowsClient
from ..requests.chat_message import ChatMessageParams
import datetime as dt
from ..types.log_status import LogStatus
from ..requests.flow_kernel_request import FlowKernelRequestParams
from ..core.request_options import RequestOptions
from ..types.create_flow_log_response import CreateFlowLogResponse
from ..types.flow_log_response import FlowLogResponse
from ..types.flow_response import FlowResponse
from ..types.project_sort_by import ProjectSortBy
from ..types.sort_order import SortOrder
from ..core.pagination import SyncPager
from ..types.paginated_data_flow_response import PaginatedDataFlowResponse
from ..core.unchecked_base_model import construct_type
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.list_flows import ListFlows
from ..types.file_environment_response import FileEnvironmentResponse
from ..requests.evaluator_activation_deactivation_request_activate_item import (
    EvaluatorActivationDeactivationRequestActivateItemParams,
)
from ..requests.evaluator_activation_deactivation_request_deactivate_item import (
    EvaluatorActivationDeactivationRequestDeactivateItemParams,
)
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawFlowsClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FlowsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFlowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFlowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFlowsClient
        """
        return self._raw_client

    def log(
        self,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        messages: typing.Optional[typing.Sequence[ChatMessageParams]] = OMIT,
        output_message: typing.Optional[ChatMessageParams] = OMIT,
        run_id: typing.Optional[str] = OMIT,
        path: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        start_time: typing.Optional[dt.datetime] = OMIT,
        end_time: typing.Optional[dt.datetime] = OMIT,
        output: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        error: typing.Optional[str] = OMIT,
        provider_latency: typing.Optional[float] = OMIT,
        stdout: typing.Optional[str] = OMIT,
        provider_request: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        provider_response: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        inputs: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        source: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        log_status: typing.Optional[LogStatus] = OMIT,
        source_datapoint_id: typing.Optional[str] = OMIT,
        trace_parent_id: typing.Optional[str] = OMIT,
        user: typing.Optional[str] = OMIT,
        flow_log_request_environment: typing.Optional[str] = OMIT,
        save: typing.Optional[bool] = OMIT,
        log_id: typing.Optional[str] = OMIT,
        flow: typing.Optional[FlowKernelRequestParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFlowLogResponse:
        """
        Log to a Flow.

        You can use query parameters `version_id`, or `environment`, to target
        an existing version of the Flow. Otherwise, the default deployed version will be chosen.

        If you create the Flow Log with a `log_status` of `incomplete`, you should later update it to `complete`
        in order to trigger Evaluators.

        Parameters
        ----------
        version_id : typing.Optional[str]
            A specific Version ID of the Flow to log to.

        environment : typing.Optional[str]
            Name of the Environment identifying a deployed version to log to.

        messages : typing.Optional[typing.Sequence[ChatMessageParams]]
            List of chat messages that were used as an input to the Flow.

        output_message : typing.Optional[ChatMessageParams]
            The output message returned by this Flow.

        run_id : typing.Optional[str]
            Unique identifier for the Run to associate the Log to.

        path : typing.Optional[str]
            Path of the Flow, including the name. This locates the Flow in the Humanloop filesystem and is used as as a unique identifier. For example: `folder/name` or just `name`.

        id : typing.Optional[str]
            ID for an existing Flow.

        start_time : typing.Optional[dt.datetime]
            The start time of the Trace. Will be updated if a child Log with an earlier start time is added.

        end_time : typing.Optional[dt.datetime]
            The end time of the Trace. Will be updated if a child Log with a later end time is added.

        output : typing.Optional[str]
            Generated output from your model for the provided inputs. Can be `None` if logging an error, or if creating a parent Log with the intention to populate it later.

        created_at : typing.Optional[dt.datetime]
            User defined timestamp for when the log was created.

        error : typing.Optional[str]
            Error message if the log is an error.

        provider_latency : typing.Optional[float]
            Duration of the logged event in seconds.

        stdout : typing.Optional[str]
            Captured log and debug statements.

        provider_request : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Raw request sent to provider.

        provider_response : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Raw response received the provider.

        inputs : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The inputs passed to the prompt template.

        source : typing.Optional[str]
            Identifies where the model was called from.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Any additional metadata to record.

        log_status : typing.Optional[LogStatus]
            Status of the Flow Log. When a Log is updated from `incomplete` to `complete`, it becomes available to Monitoring Evaluators. Flow Logs cannot have an unspecified status: they must start as `incomplete` to allow children to be added. Provide `complete` if you don't intend to add children to the trace.

        source_datapoint_id : typing.Optional[str]
            Unique identifier for the Datapoint that this Log is derived from. This can be used by Humanloop to associate Logs to Evaluations. If provided, Humanloop will automatically associate this Log to Evaluations that require a Log for this Datapoint-Version pair.

        trace_parent_id : typing.Optional[str]
            The ID of the parent Log to nest this Log under in a Trace.

        user : typing.Optional[str]
            End-user ID related to the Log.

        flow_log_request_environment : typing.Optional[str]
            The name of the Environment the Log is associated to.

        save : typing.Optional[bool]
            Whether the request/response payloads will be stored on Humanloop.

        log_id : typing.Optional[str]
            This will identify a Log. If you don't provide a Log ID, Humanloop will generate one for you.

        flow : typing.Optional[FlowKernelRequestParams]
            Flow used to generate the Trace.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFlowLogResponse
            Successful Response

        Examples
        --------
        import datetime

        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.log(
            id="fl_6o701g4jmcanPVHxdqD0O",
            flow={
                "attributes": {
                    "prompt": {
                        "template": "You are a helpful assistant helping with medical anamnesis",
                        "model": "gpt-4o",
                        "temperature": 0.8,
                    },
                    "tool": {
                        "name": "retrieval_tool_v3",
                        "description": "Retrieval tool for MedQA.",
                        "source_code": "def retrieval_tool(question: str) -> str:\n    pass\n",
                    },
                }
            },
            inputs={
                "question": "Patient with a history of diabetes and hypertension presents with chest pain and shortness of breath."
            },
            output="The patient is likely experiencing a myocardial infarction. Immediate medical attention is required.",
            log_status="incomplete",
            start_time=datetime.datetime.fromisoformat(
                "2024-07-08 19:40:35+00:00",
            ),
            end_time=datetime.datetime.fromisoformat(
                "2024-07-08 19:40:39+00:00",
            ),
        )
        """
        response = self._raw_client.log(
            version_id=version_id,
            environment=environment,
            messages=messages,
            output_message=output_message,
            run_id=run_id,
            path=path,
            id=id,
            start_time=start_time,
            end_time=end_time,
            output=output,
            created_at=created_at,
            error=error,
            provider_latency=provider_latency,
            stdout=stdout,
            provider_request=provider_request,
            provider_response=provider_response,
            inputs=inputs,
            source=source,
            metadata=metadata,
            log_status=log_status,
            source_datapoint_id=source_datapoint_id,
            trace_parent_id=trace_parent_id,
            user=user,
            flow_log_request_environment=flow_log_request_environment,
            save=save,
            log_id=log_id,
            flow=flow,
            request_options=request_options,
        )
        return response.data

    def update_log(
        self,
        log_id: str,
        *,
        messages: typing.Optional[typing.Sequence[ChatMessageParams]] = OMIT,
        output_message: typing.Optional[ChatMessageParams] = OMIT,
        inputs: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        output: typing.Optional[str] = OMIT,
        error: typing.Optional[str] = OMIT,
        log_status: typing.Optional[LogStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowLogResponse:
        """
        Update the status, inputs, output of a Flow Log.

        Marking a Flow Log as complete will trigger any monitoring Evaluators to run.
        Inputs and output (or error) must be provided in order to mark it as complete.

        The end_time log attribute will be set to match the time the log is marked as complete.

        Parameters
        ----------
        log_id : str
            Unique identifier of the Flow Log.

        messages : typing.Optional[typing.Sequence[ChatMessageParams]]
            List of chat messages that were used as an input to the Flow.

        output_message : typing.Optional[ChatMessageParams]
            The output message returned by this Flow.

        inputs : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The inputs passed to the Flow Log.

        output : typing.Optional[str]
            The output of the Flow Log. Provide None to unset existing `output` value. Provide either this, `output_message` or `error`.

        error : typing.Optional[str]
            The error message of the Flow Log. Provide None to unset existing `error` value. Provide either this, `output_message` or `output`.

        log_status : typing.Optional[LogStatus]
            Status of the Flow Log. When a Flow Log is updated to `complete`, no more Logs can be added to it. Monitoring Evaluators will only run on `complete` Flow Logs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowLogResponse
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.update_log(
            log_id="medqa_experiment_0001",
            inputs={
                "question": "Patient with a history of diabetes and normal tension presents with chest pain and shortness of breath."
            },
            output="The patient is likely experiencing a myocardial infarction. Immediate medical attention is required.",
            log_status="complete",
        )
        """
        response = self._raw_client.update_log(
            log_id,
            messages=messages,
            output_message=output_message,
            inputs=inputs,
            output=output,
            error=error,
            log_status=log_status,
            request_options=request_options,
        )
        return response.data

    def get(
        self,
        id: str,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Retrieve the Flow with the given ID.

        By default, the deployed version of the Flow is returned. Use the query parameters
        `version_id` or `environment` to target a specific version of the Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        version_id : typing.Optional[str]
            A specific Version ID of the Flow to retrieve.

        environment : typing.Optional[str]
            Name of the Environment to retrieve a deployed Version from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.get(
            id="fl_6o701g4jmcanPVHxdqD0O",
        )
        """
        response = self._raw_client.get(
            id, version_id=version_id, environment=environment, request_options=request_options
        )
        return response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the Flow with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.delete(
            id="fl_6o701g4jmcanPVHxdqD0O",
        )
        """
        response = self._raw_client.delete(id, request_options=request_options)
        return response.data

    def move(
        self,
        id: str,
        *,
        path: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        directory_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Move the Flow to a different path or change the name.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        path : typing.Optional[str]
            Path of the Flow including the Flow name, which is used as a unique identifier.

        name : typing.Optional[str]
            Name of the Flow.

        directory_id : typing.Optional[str]
            Unique identifier for the Directory to move Flow to. Starts with `dir_`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.move(
            id="fl_6o701g4jmcanPVHxdqD0O",
            path="new directory/new name",
        )
        """
        response = self._raw_client.move(
            id, path=path, name=name, directory_id=directory_id, request_options=request_options
        )
        return response.data

    def list(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        user_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ProjectSortBy] = None,
        order: typing.Optional[SortOrder] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[FlowResponse]:
        """
        Get a list of Flows.

        Parameters
        ----------
        page : typing.Optional[int]
            Page number for pagination.

        size : typing.Optional[int]
            Page size for pagination. Number of Flows to fetch.

        name : typing.Optional[str]
            Case-insensitive filter for Flow name.

        user_filter : typing.Optional[str]
            Case-insensitive filter for users in the Flow. This filter matches against both email address and name of users.

        sort_by : typing.Optional[ProjectSortBy]
            Field to sort Flows by

        order : typing.Optional[SortOrder]
            Direction to sort by.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[FlowResponse]
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        response = client.flows.list(
            size=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        page = page if page is not None else 1
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "flows",
            method="GET",
            params={
                "page": page,
                "size": size,
                "name": name,
                "user_filter": user_filter,
                "sort_by": sort_by,
                "order": order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    PaginatedDataFlowResponse,
                    construct_type(
                        type_=PaginatedDataFlowResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _has_next = True
                _get_next = lambda: self.list(
                    page=page + 1,
                    size=size,
                    name=name,
                    user_filter=user_filter,
                    sort_by=sort_by,
                    order=order,
                    request_options=request_options,
                )
                _items = _parsed_response.records
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert(
        self,
        *,
        attributes: typing.Dict[str, typing.Optional[typing.Any]],
        path: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        version_name: typing.Optional[str] = OMIT,
        version_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Create or update a Flow.

        Flows can also be identified by the `ID` or their `path`.

        You can provide `version_name` and `version_description` to identify and describe your versions.
        Version names must be unique within a Flow - attempting to create a version with a name
        that already exists will result in a 409 Conflict error.

        Parameters
        ----------
        attributes : typing.Dict[str, typing.Optional[typing.Any]]
            A key-value object identifying the Flow Version.

        path : typing.Optional[str]
            Path of the Flow, including the name. This locates the Flow in the Humanloop filesystem and is used as as a unique identifier. For example: `folder/name` or just `name`.

        id : typing.Optional[str]
            ID for an existing Flow.

        version_name : typing.Optional[str]
            Unique name for the Flow version. Version names must be unique for a given Flow.

        version_description : typing.Optional[str]
            Description of the version, e.g., the changes made in this version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.upsert(
            path="Personal Projects/MedQA Flow",
            attributes={
                "prompt": {
                    "template": "You are a helpful medical assistant helping with medical anamnesis. Answer {{question}}",
                    "model": "gpt-4o",
                    "temperature": 0.8,
                },
                "tool": {
                    "name": "retrieval_tool_v3",
                    "description": "Retrieval tool for MedQA.",
                    "source_code": "def retrieval_tool(question: str) -> str:\n    pass\n",
                },
                "version_name": "medqa-flow-v1",
                "version_description": "Initial version",
            },
        )
        """
        response = self._raw_client.upsert(
            attributes=attributes,
            path=path,
            id=id,
            version_name=version_name,
            version_description=version_description,
            request_options=request_options,
        )
        return response.data

    def list_versions(
        self,
        id: str,
        *,
        evaluator_aggregates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListFlows:
        """
        Get a list of all the versions of a Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        evaluator_aggregates : typing.Optional[bool]
            Whether to include Evaluator aggregate results for the versions in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListFlows
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.list_versions(
            id="fl_6o701g4jmcanPVHxdqD0O",
        )
        """
        response = self._raw_client.list_versions(
            id, evaluator_aggregates=evaluator_aggregates, request_options=request_options
        )
        return response.data

    def delete_flow_version(
        self, id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a version of the Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        version_id : str
            Unique identifier for the specific version of the Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.delete_flow_version(
            id="id",
            version_id="version_id",
        )
        """
        response = self._raw_client.delete_flow_version(id, version_id, request_options=request_options)
        return response.data

    def update_flow_version(
        self,
        id: str,
        version_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Update the name or description of the Flow version.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        version_id : str
            Unique identifier for the specific version of the Flow.

        name : typing.Optional[str]
            Name of the version.

        description : typing.Optional[str]
            Description of the version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.update_flow_version(
            id="id",
            version_id="version_id",
        )
        """
        response = self._raw_client.update_flow_version(
            id, version_id, name=name, description=description, request_options=request_options
        )
        return response.data

    def set_deployment(
        self, id: str, environment_id: str, *, version_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FlowResponse:
        """
        Deploy Flow to an Environment.

        Set the deployed version for the specified Environment. This Flow
        will be used for calls made to the Flow in this Environment.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        environment_id : str
            Unique identifier for the Environment to deploy the Version to.

        version_id : str
            Unique identifier for the specific version of the Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.set_deployment(
            id="fl_6o701g4jmcanPVHxdqD0O",
            environment_id="staging",
            version_id="flv_6o701g4jmcanPVHxdqD0O",
        )
        """
        response = self._raw_client.set_deployment(
            id, environment_id, version_id=version_id, request_options=request_options
        )
        return response.data

    def remove_deployment(
        self, id: str, environment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove deployed Flow from the Environment.

        Remove the deployed version for the specified Environment. This Flow
        will no longer be used for calls made to the Flow in this Environment.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        environment_id : str
            Unique identifier for the Environment to remove the deployment from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.remove_deployment(
            id="fl_6o701g4jmcanPVHxdqD0O",
            environment_id="staging",
        )
        """
        response = self._raw_client.remove_deployment(id, environment_id, request_options=request_options)
        return response.data

    def list_environments(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[FileEnvironmentResponse]:
        """
        List all Environments and their deployed versions for the Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[FileEnvironmentResponse]
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.list_environments(
            id="fl_6o701g4jmcanPVHxdqD0O",
        )
        """
        response = self._raw_client.list_environments(id, request_options=request_options)
        return response.data

    def update_monitoring(
        self,
        id: str,
        *,
        activate: typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestActivateItemParams]] = OMIT,
        deactivate: typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestDeactivateItemParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Activate and deactivate Evaluators for monitoring the Flow.

        An activated Evaluator will automatically be run on all new "completed" Logs
        within the Flow for monitoring purposes.

        Parameters
        ----------
        id : str

        activate : typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestActivateItemParams]]
            Evaluators to activate for Monitoring. These will be automatically run on new Logs.

        deactivate : typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestDeactivateItemParams]]
            Evaluators to deactivate. These will not be run on new Logs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        from humanloop import Humanloop

        client = Humanloop(
            api_key="YOUR_API_KEY",
        )
        client.flows.update_monitoring(
            id="fl_6o701g4jmcanPVHxdqD0O",
            activate=[{"evaluator_version_id": "evv_1abc4308abd"}],
        )
        """
        response = self._raw_client.update_monitoring(
            id, activate=activate, deactivate=deactivate, request_options=request_options
        )
        return response.data


class AsyncFlowsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFlowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFlowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFlowsClient
        """
        return self._raw_client

    async def log(
        self,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        messages: typing.Optional[typing.Sequence[ChatMessageParams]] = OMIT,
        output_message: typing.Optional[ChatMessageParams] = OMIT,
        run_id: typing.Optional[str] = OMIT,
        path: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        start_time: typing.Optional[dt.datetime] = OMIT,
        end_time: typing.Optional[dt.datetime] = OMIT,
        output: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        error: typing.Optional[str] = OMIT,
        provider_latency: typing.Optional[float] = OMIT,
        stdout: typing.Optional[str] = OMIT,
        provider_request: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        provider_response: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        inputs: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        source: typing.Optional[str] = OMIT,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        log_status: typing.Optional[LogStatus] = OMIT,
        source_datapoint_id: typing.Optional[str] = OMIT,
        trace_parent_id: typing.Optional[str] = OMIT,
        user: typing.Optional[str] = OMIT,
        flow_log_request_environment: typing.Optional[str] = OMIT,
        save: typing.Optional[bool] = OMIT,
        log_id: typing.Optional[str] = OMIT,
        flow: typing.Optional[FlowKernelRequestParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateFlowLogResponse:
        """
        Log to a Flow.

        You can use query parameters `version_id`, or `environment`, to target
        an existing version of the Flow. Otherwise, the default deployed version will be chosen.

        If you create the Flow Log with a `log_status` of `incomplete`, you should later update it to `complete`
        in order to trigger Evaluators.

        Parameters
        ----------
        version_id : typing.Optional[str]
            A specific Version ID of the Flow to log to.

        environment : typing.Optional[str]
            Name of the Environment identifying a deployed version to log to.

        messages : typing.Optional[typing.Sequence[ChatMessageParams]]
            List of chat messages that were used as an input to the Flow.

        output_message : typing.Optional[ChatMessageParams]
            The output message returned by this Flow.

        run_id : typing.Optional[str]
            Unique identifier for the Run to associate the Log to.

        path : typing.Optional[str]
            Path of the Flow, including the name. This locates the Flow in the Humanloop filesystem and is used as as a unique identifier. For example: `folder/name` or just `name`.

        id : typing.Optional[str]
            ID for an existing Flow.

        start_time : typing.Optional[dt.datetime]
            The start time of the Trace. Will be updated if a child Log with an earlier start time is added.

        end_time : typing.Optional[dt.datetime]
            The end time of the Trace. Will be updated if a child Log with a later end time is added.

        output : typing.Optional[str]
            Generated output from your model for the provided inputs. Can be `None` if logging an error, or if creating a parent Log with the intention to populate it later.

        created_at : typing.Optional[dt.datetime]
            User defined timestamp for when the log was created.

        error : typing.Optional[str]
            Error message if the log is an error.

        provider_latency : typing.Optional[float]
            Duration of the logged event in seconds.

        stdout : typing.Optional[str]
            Captured log and debug statements.

        provider_request : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Raw request sent to provider.

        provider_response : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Raw response received the provider.

        inputs : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The inputs passed to the prompt template.

        source : typing.Optional[str]
            Identifies where the model was called from.

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            Any additional metadata to record.

        log_status : typing.Optional[LogStatus]
            Status of the Flow Log. When a Log is updated from `incomplete` to `complete`, it becomes available to Monitoring Evaluators. Flow Logs cannot have an unspecified status: they must start as `incomplete` to allow children to be added. Provide `complete` if you don't intend to add children to the trace.

        source_datapoint_id : typing.Optional[str]
            Unique identifier for the Datapoint that this Log is derived from. This can be used by Humanloop to associate Logs to Evaluations. If provided, Humanloop will automatically associate this Log to Evaluations that require a Log for this Datapoint-Version pair.

        trace_parent_id : typing.Optional[str]
            The ID of the parent Log to nest this Log under in a Trace.

        user : typing.Optional[str]
            End-user ID related to the Log.

        flow_log_request_environment : typing.Optional[str]
            The name of the Environment the Log is associated to.

        save : typing.Optional[bool]
            Whether the request/response payloads will be stored on Humanloop.

        log_id : typing.Optional[str]
            This will identify a Log. If you don't provide a Log ID, Humanloop will generate one for you.

        flow : typing.Optional[FlowKernelRequestParams]
            Flow used to generate the Trace.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateFlowLogResponse
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.log(
                id="fl_6o701g4jmcanPVHxdqD0O",
                flow={
                    "attributes": {
                        "prompt": {
                            "template": "You are a helpful assistant helping with medical anamnesis",
                            "model": "gpt-4o",
                            "temperature": 0.8,
                        },
                        "tool": {
                            "name": "retrieval_tool_v3",
                            "description": "Retrieval tool for MedQA.",
                            "source_code": "def retrieval_tool(question: str) -> str:\n    pass\n",
                        },
                    }
                },
                inputs={
                    "question": "Patient with a history of diabetes and hypertension presents with chest pain and shortness of breath."
                },
                output="The patient is likely experiencing a myocardial infarction. Immediate medical attention is required.",
                log_status="incomplete",
                start_time=datetime.datetime.fromisoformat(
                    "2024-07-08 19:40:35+00:00",
                ),
                end_time=datetime.datetime.fromisoformat(
                    "2024-07-08 19:40:39+00:00",
                ),
            )


        asyncio.run(main())
        """
        response = await self._raw_client.log(
            version_id=version_id,
            environment=environment,
            messages=messages,
            output_message=output_message,
            run_id=run_id,
            path=path,
            id=id,
            start_time=start_time,
            end_time=end_time,
            output=output,
            created_at=created_at,
            error=error,
            provider_latency=provider_latency,
            stdout=stdout,
            provider_request=provider_request,
            provider_response=provider_response,
            inputs=inputs,
            source=source,
            metadata=metadata,
            log_status=log_status,
            source_datapoint_id=source_datapoint_id,
            trace_parent_id=trace_parent_id,
            user=user,
            flow_log_request_environment=flow_log_request_environment,
            save=save,
            log_id=log_id,
            flow=flow,
            request_options=request_options,
        )
        return response.data

    async def update_log(
        self,
        log_id: str,
        *,
        messages: typing.Optional[typing.Sequence[ChatMessageParams]] = OMIT,
        output_message: typing.Optional[ChatMessageParams] = OMIT,
        inputs: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        output: typing.Optional[str] = OMIT,
        error: typing.Optional[str] = OMIT,
        log_status: typing.Optional[LogStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowLogResponse:
        """
        Update the status, inputs, output of a Flow Log.

        Marking a Flow Log as complete will trigger any monitoring Evaluators to run.
        Inputs and output (or error) must be provided in order to mark it as complete.

        The end_time log attribute will be set to match the time the log is marked as complete.

        Parameters
        ----------
        log_id : str
            Unique identifier of the Flow Log.

        messages : typing.Optional[typing.Sequence[ChatMessageParams]]
            List of chat messages that were used as an input to the Flow.

        output_message : typing.Optional[ChatMessageParams]
            The output message returned by this Flow.

        inputs : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
            The inputs passed to the Flow Log.

        output : typing.Optional[str]
            The output of the Flow Log. Provide None to unset existing `output` value. Provide either this, `output_message` or `error`.

        error : typing.Optional[str]
            The error message of the Flow Log. Provide None to unset existing `error` value. Provide either this, `output_message` or `output`.

        log_status : typing.Optional[LogStatus]
            Status of the Flow Log. When a Flow Log is updated to `complete`, no more Logs can be added to it. Monitoring Evaluators will only run on `complete` Flow Logs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowLogResponse
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.update_log(
                log_id="medqa_experiment_0001",
                inputs={
                    "question": "Patient with a history of diabetes and normal tension presents with chest pain and shortness of breath."
                },
                output="The patient is likely experiencing a myocardial infarction. Immediate medical attention is required.",
                log_status="complete",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update_log(
            log_id,
            messages=messages,
            output_message=output_message,
            inputs=inputs,
            output=output,
            error=error,
            log_status=log_status,
            request_options=request_options,
        )
        return response.data

    async def get(
        self,
        id: str,
        *,
        version_id: typing.Optional[str] = None,
        environment: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Retrieve the Flow with the given ID.

        By default, the deployed version of the Flow is returned. Use the query parameters
        `version_id` or `environment` to target a specific version of the Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        version_id : typing.Optional[str]
            A specific Version ID of the Flow to retrieve.

        environment : typing.Optional[str]
            Name of the Environment to retrieve a deployed Version from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.get(
                id="fl_6o701g4jmcanPVHxdqD0O",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(
            id, version_id=version_id, environment=environment, request_options=request_options
        )
        return response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the Flow with the given ID.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.delete(
                id="fl_6o701g4jmcanPVHxdqD0O",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(id, request_options=request_options)
        return response.data

    async def move(
        self,
        id: str,
        *,
        path: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        directory_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Move the Flow to a different path or change the name.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        path : typing.Optional[str]
            Path of the Flow including the Flow name, which is used as a unique identifier.

        name : typing.Optional[str]
            Name of the Flow.

        directory_id : typing.Optional[str]
            Unique identifier for the Directory to move Flow to. Starts with `dir_`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.move(
                id="fl_6o701g4jmcanPVHxdqD0O",
                path="new directory/new name",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.move(
            id, path=path, name=name, directory_id=directory_id, request_options=request_options
        )
        return response.data

    async def list(
        self,
        *,
        page: typing.Optional[int] = None,
        size: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        user_filter: typing.Optional[str] = None,
        sort_by: typing.Optional[ProjectSortBy] = None,
        order: typing.Optional[SortOrder] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[FlowResponse]:
        """
        Get a list of Flows.

        Parameters
        ----------
        page : typing.Optional[int]
            Page number for pagination.

        size : typing.Optional[int]
            Page size for pagination. Number of Flows to fetch.

        name : typing.Optional[str]
            Case-insensitive filter for Flow name.

        user_filter : typing.Optional[str]
            Case-insensitive filter for users in the Flow. This filter matches against both email address and name of users.

        sort_by : typing.Optional[ProjectSortBy]
            Field to sort Flows by

        order : typing.Optional[SortOrder]
            Direction to sort by.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[FlowResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.flows.list(
                size=1,
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        page = page if page is not None else 1
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "flows",
            method="GET",
            params={
                "page": page,
                "size": size,
                "name": name,
                "user_filter": user_filter,
                "sort_by": sort_by,
                "order": order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    PaginatedDataFlowResponse,
                    construct_type(
                        type_=PaginatedDataFlowResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _has_next = True
                _get_next = lambda: self.list(
                    page=page + 1,
                    size=size,
                    name=name,
                    user_filter=user_filter,
                    sort_by=sort_by,
                    order=order,
                    request_options=request_options,
                )
                _items = _parsed_response.records
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert(
        self,
        *,
        attributes: typing.Dict[str, typing.Optional[typing.Any]],
        path: typing.Optional[str] = OMIT,
        id: typing.Optional[str] = OMIT,
        version_name: typing.Optional[str] = OMIT,
        version_description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Create or update a Flow.

        Flows can also be identified by the `ID` or their `path`.

        You can provide `version_name` and `version_description` to identify and describe your versions.
        Version names must be unique within a Flow - attempting to create a version with a name
        that already exists will result in a 409 Conflict error.

        Parameters
        ----------
        attributes : typing.Dict[str, typing.Optional[typing.Any]]
            A key-value object identifying the Flow Version.

        path : typing.Optional[str]
            Path of the Flow, including the name. This locates the Flow in the Humanloop filesystem and is used as as a unique identifier. For example: `folder/name` or just `name`.

        id : typing.Optional[str]
            ID for an existing Flow.

        version_name : typing.Optional[str]
            Unique name for the Flow version. Version names must be unique for a given Flow.

        version_description : typing.Optional[str]
            Description of the version, e.g., the changes made in this version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.upsert(
                path="Personal Projects/MedQA Flow",
                attributes={
                    "prompt": {
                        "template": "You are a helpful medical assistant helping with medical anamnesis. Answer {{question}}",
                        "model": "gpt-4o",
                        "temperature": 0.8,
                    },
                    "tool": {
                        "name": "retrieval_tool_v3",
                        "description": "Retrieval tool for MedQA.",
                        "source_code": "def retrieval_tool(question: str) -> str:\n    pass\n",
                    },
                    "version_name": "medqa-flow-v1",
                    "version_description": "Initial version",
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.upsert(
            attributes=attributes,
            path=path,
            id=id,
            version_name=version_name,
            version_description=version_description,
            request_options=request_options,
        )
        return response.data

    async def list_versions(
        self,
        id: str,
        *,
        evaluator_aggregates: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListFlows:
        """
        Get a list of all the versions of a Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        evaluator_aggregates : typing.Optional[bool]
            Whether to include Evaluator aggregate results for the versions in the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListFlows
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.list_versions(
                id="fl_6o701g4jmcanPVHxdqD0O",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.list_versions(
            id, evaluator_aggregates=evaluator_aggregates, request_options=request_options
        )
        return response.data

    async def delete_flow_version(
        self, id: str, version_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a version of the Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        version_id : str
            Unique identifier for the specific version of the Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.delete_flow_version(
                id="id",
                version_id="version_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete_flow_version(id, version_id, request_options=request_options)
        return response.data

    async def update_flow_version(
        self,
        id: str,
        version_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Update the name or description of the Flow version.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        version_id : str
            Unique identifier for the specific version of the Flow.

        name : typing.Optional[str]
            Name of the version.

        description : typing.Optional[str]
            Description of the version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.update_flow_version(
                id="id",
                version_id="version_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update_flow_version(
            id, version_id, name=name, description=description, request_options=request_options
        )
        return response.data

    async def set_deployment(
        self, id: str, environment_id: str, *, version_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> FlowResponse:
        """
        Deploy Flow to an Environment.

        Set the deployed version for the specified Environment. This Flow
        will be used for calls made to the Flow in this Environment.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        environment_id : str
            Unique identifier for the Environment to deploy the Version to.

        version_id : str
            Unique identifier for the specific version of the Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.set_deployment(
                id="fl_6o701g4jmcanPVHxdqD0O",
                environment_id="staging",
                version_id="flv_6o701g4jmcanPVHxdqD0O",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.set_deployment(
            id, environment_id, version_id=version_id, request_options=request_options
        )
        return response.data

    async def remove_deployment(
        self, id: str, environment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove deployed Flow from the Environment.

        Remove the deployed version for the specified Environment. This Flow
        will no longer be used for calls made to the Flow in this Environment.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        environment_id : str
            Unique identifier for the Environment to remove the deployment from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.remove_deployment(
                id="fl_6o701g4jmcanPVHxdqD0O",
                environment_id="staging",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.remove_deployment(id, environment_id, request_options=request_options)
        return response.data

    async def list_environments(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[FileEnvironmentResponse]:
        """
        List all Environments and their deployed versions for the Flow.

        Parameters
        ----------
        id : str
            Unique identifier for Flow.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[FileEnvironmentResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.list_environments(
                id="fl_6o701g4jmcanPVHxdqD0O",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.list_environments(id, request_options=request_options)
        return response.data

    async def update_monitoring(
        self,
        id: str,
        *,
        activate: typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestActivateItemParams]] = OMIT,
        deactivate: typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestDeactivateItemParams]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FlowResponse:
        """
        Activate and deactivate Evaluators for monitoring the Flow.

        An activated Evaluator will automatically be run on all new "completed" Logs
        within the Flow for monitoring purposes.

        Parameters
        ----------
        id : str

        activate : typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestActivateItemParams]]
            Evaluators to activate for Monitoring. These will be automatically run on new Logs.

        deactivate : typing.Optional[typing.Sequence[EvaluatorActivationDeactivationRequestDeactivateItemParams]]
            Evaluators to deactivate. These will not be run on new Logs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FlowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from humanloop import AsyncHumanloop

        client = AsyncHumanloop(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.flows.update_monitoring(
                id="fl_6o701g4jmcanPVHxdqD0O",
                activate=[{"evaluator_version_id": "evv_1abc4308abd"}],
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update_monitoring(
            id, activate=activate, deactivate=deactivate, request_options=request_options
        )
        return response.data
